<?php

namespace FiloBlu\OrderStatus\Observer;

use \Magento\Framework\Event\ObserverInterface;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use \Magento\Store\Model\ScopeInterface;

/**
 * Observer on Amazon Pay authorization and capture and set order status accordingly
 * This observer needs the patch amzn-pay-module-events.patch
 */
 
 class UpdateStatusAmazonPay implements ObserverInterface {
    
    /**
     * @var \Psr\Log\LoggerInterface $logger
     */  
    protected $_logger;
    
    protected $_config;
    
    const CAPTURE_ORDER_STATUS = 'payment_confirmed';
    const AUTHORIZE_ORDER_STATUS = 'confirmed';
    const CAPTURE_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_authorized';
    const AUTHORIZE_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_pre_authorized';
    const DEBUG_LOG_ACTIVE = TRUE;
	
    /**
     * @param \Psr\Log\LoggerInterface $logger
     */  
	public function __construct(
		\Psr\Log\LoggerInterface $logger,
                ScopeConfigInterface $config
	){
		$this->_logger = $logger;
                $this->_config = $config;
	}

    /**
     * Process order
     *
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(Observer $observer){
        /* @var $order \Magento\Sales\Model\Order */
        $event = $observer->getEvent();
        if (self::DEBUG_LOG_ACTIVE) {
            $this->_logger->addInfo("FiloBlu\UpdateStatusAmazonPay\Observer: Processing IPN event: " . $event);
        }
        $order = $observer->getOrder();
        switch ($event) {
            case 'complete_pending_capture':
                $captured_order_status = $this->_config->getValue(self::CAPTURE_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ? 
                $this->_config->getValue(self::CAPTURE_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::CAPTURE_ORDER_STATUS;
                $order->addStatusHistoryComment(
                    __('Order processed by mod2-order-status'), $captured_order_status
                );
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->addInfo("FiloBlu\UpdateStatusAmazonPay\Observer: Order status has been set to " . $captured_order_status);
                }
                break;
            case 'complete_pending_authorization':
                $authorized_order_status = $this->_config->getValue(self::AUTHORIZE_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ? 
                        $this->_config->getValue(self::AUTHORIZE_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::AUTHORIZE_ORDER_STATUS;
                $order->addStatusHistoryComment(
                    __('Order processed by mod2-order-status'), $authorized_order_status
                );
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->addInfo("FiloBlu\UpdateStatusAmazonPay\Observer: Order status has been set to " . $authorized_order_status);
                }
                break;
        }
        return TRUE;
    }
}